@@ -1,6 +1,7 @@
 import hdroller.math
 import hdroller.die_repeater
 
+from collections import defaultdict
 from functools import cached_property, lru_cache
 import numpy
 import re
@@ -59,19 +60,30 @@ class Die(metaclass=DieType):
             if min_outcome is not None: raise ValueError()
             self._weights = numpy.array([1.0])
             self._min_outcome = weights
-        elif numpy.issubdtype(type(weights), numpy.floating):
-            if min_outcome is not None: raise ValueError()
-            if not (weights >= 0.0 and weights <= 1.0):
-                raise ValueError('Only floats between 0.0 and 1.0 can be cast to Die.')
-            self._weights = numpy.array([1.0 - weights, weights])
-            self._min_outcome = 0
+        elif hasattr(weights, 'items'):
+            # A dict mapping outcomes to weights.
+            self._min_outcome = min(weights.keys())
+            max_outcome = max(weights.keys())
+            self._weights = numpy.zeros((max_outcome - self._min_outcome + 1,))
+            for outcome, weight in weights.items():
+                self._weights[outcome - self._min_outcome] = weight
         else:
+            if min_outcome is None:
+                raise TypeError('When Die is constructed with a weights array, min_outcome must be provided.')
             # weights is an array.
             if not numpy.issubdtype(type(min_outcome), numpy.integer):
                 raise ValueError('min_outcome must be of integer type')
-            self._weights = numpy.array(weights)
+            if numpy.any(numpy.isinf(weights)):
+                raise ValueError('Weights must be finite.')
+            self._weights = numpy.array(weights, dtype=float)
+            if self._weights.ndim != 1:
+                raise ValueError('Weights must have exactly one dimension.')
             self._min_outcome = min_outcome
         
+        total_weight = numpy.sum(self._weights)
+        if total_weight >= hdroller.math.MAX_INT_FLOAT:
+            self._weights /= total_weight
+        
         self._weights.setflags(write=False)
     
     # Cached values.
@@ -107,15 +119,13 @@ class Die(metaclass=DieType):
     
     @cached_property
     def _cdf(self):
-        result = numpy.cumsum(self.pmf())
-        result = numpy.insert(result, 0, 0.0)
+        result = self._cweights / self.total_weight()
         result.setflags(write=False)
         return result
     
     @cached_property
     def _ccdf(self):
-        result = hdroller.math.reverse_cumsum(self.pmf())
-        result = numpy.append(result, 0.0)
+        result = self._ccweights / self.total_weight()
         result.setflags(write=False)
         return result
     
@@ -124,33 +134,6 @@ class Die(metaclass=DieType):
         return hdroller.die_repeater.DieRepeater(self)
     
     # Creation.
-
-    @staticmethod
-    def mix(*args, mix_weights=None):
-        """
-        Constructs a Die from a mixture of the arguments,
-        equivalent to rolling a die and then choosing one of the arguments
-        based on the resulting face rolled.
-        The arguments can be Dice or anything castable to Dice.
-        mix_weights: An array of one weight per argument.
-          If not provided, all arguments are mixed uniformly.
-          A Die can also be used, in which case its weights determines the mix_weights.
-        """
-        if len(args) == 1 and not isinstance(args[0], Die):
-            args = args[0]
-        
-        args = [Die(die) for die in args]
-        args = Die._union(*args)
-        
-        if mix_weights is None:
-            mix_weights = numpy.ones((len(args),))
-        elif isinstance(mix_weights, Die):
-            mix_weights = mix_weights.weights()
-
-        weights = numpy.zeros_like(args[0].weights())
-        for die, mix_weight in zip(args, mix_weights):
-            weights += mix_weight * die.weights()
-        return Die(weights, args[0].min_outcome())
     
     @staticmethod
     @lru_cache(maxsize=None)
@@ -160,8 +143,8 @@ class Die(metaclass=DieType):
     
     # TODO: Apply cache to all __new__ dice created with floats?
     @staticmethod
-    def bernoulli(chance=0.5):
-        return Die(chance)
+    def bernoulli(chance=0.5, denominator=1.0):
+        return Die([denominator - chance, chance], 0)
 
     coin = bernoulli
     
@@ -198,21 +181,7 @@ class Die(metaclass=DieType):
             num_dice = Die(args[0])
 
         return num_dice * tail_die
-
-    @staticmethod
-    def geometric(max_outcome=100, **kwargs):
-        """
-        A truncated geometric distribution.
-        Any remaining probability is placed on max_outcome.
-        """
-        if 'half_life' in kwargs:
-            factor = 0.5 ** (1.0 / kwargs['half_life'])
-        else:
-            raise RuntimeError('Geometric distribution requires a scaling parameter.')
-        pmf = numpy.power(factor, numpy.arange(max_outcome+1)) * (1.0 - factor)
-        pmf[-1] = 1.0 - numpy.sum(pmf[:-1])
-        return Die(pmf, 0)
-        
+     
     @staticmethod
     def from_cweights(cweights, min_outcome, total_weight_if_exclusive=None):
         """
@@ -253,6 +222,26 @@ class Die(metaclass=DieType):
             pmf = numpy.flip(numpy.diff(numpy.flip(ccdf), append=1.0))
         return Die(pmf, min_outcome)
     
+    @staticmethod
+    def from_rv(rv, min_outcome, max_outcome, **kwargs):
+        """
+        Constructs a die by discretizing a supplied rv object (as scipy.stats).
+        Any additional arguments are provided to rv.cdf().
+        
+        For example,
+        Die.from_rv(scipy.stats.norm, -40, 40, scale=10)
+        would discretize a normal distribution with a standard deviation of 10.
+        
+        Values beyond min/max outcome are clipped.
+        """
+        x = numpy.arange(min_outcome, max_outcome)
+        if hasattr(rv, 'pdf'):
+            # Continuous distributions get rounded.
+            x = x + 0.5
+        cdf = rv.cdf(x, **kwargs)
+        cdf = numpy.append(cdf, 1.0)
+        return Die.from_cdf(cdf, min_outcome)
+    
     # Outcome information.
     def __len__(self):
         return len(self._weights)
@@ -265,14 +254,29 @@ class Die(metaclass=DieType):
 
     def outcomes(self, prepend=False, append=False):
         return numpy.array(range(self.min_outcome(), self.min_outcome() + len(self) + (append is not False))) + (prepend is not False)
+        
+    def probability(self, outcome):
+        """
+        Returns the probability of a single outcome.
+        """
+        if outcome < self.min_outcome() or outcome > self.max_outcome():
+            return 0.0
+        else:
+            return self.pmf()[outcome - self.min_outcome()]
 
     # Distributions.
-    def weights(self):
-        return self._weights
         
     def is_exact(self):
         return self._is_exact
     
+    def is_bernoulli(self):
+        return self.min_outcome() >= 0 and self.max_outcome() <= 1
+    
+    is_coin = is_bernoulli
+        
+    def weights(self):
+        return self._weights
+    
     def pmf(self):
         return self._pmf
         
@@ -379,12 +383,12 @@ class Die(metaclass=DieType):
         
     def ks_stat(self, other):
         """ Kolmogorov–Smirnov stat. The maximum absolute difference between CDFs. """
-        a, b = Die._union(self, other, lcd=False)
+        a, b = Die._align([self, other], lcd=False)
         return numpy.max(numpy.abs(a.cdf() - b.cdf()))
     
     def cvm_stat(self, other):
         """ Cramér-von Mises stat. The sum-of-squares difference between CDFs. """
-        a, b = Die._union(self, other, lcd=False)
+        a, b = Die._align([self, other], lcd=False)
         return numpy.linalg.norm(a.cdf() - b.cdf())
     
     def total_weight(self):
@@ -395,86 +399,25 @@ class Die(metaclass=DieType):
     def __neg__(self):
         """ Returns a Die with all outcomes negated. """
         return Die(numpy.flip(self.weights()), -self.max_outcome())
+        
+    def __abs__(self):
+        """ Take the absolute value of all outcomes. """
+        if self.min_outcome() >= 0: return self
+        if self.max_outcome() <= 0: return -self
+        
+        # If the die doesn't fall into either of the simple cases above,
+        # it crosses zero.
+        max_outcome = max(-self.min_outcome(), self.max_outcome())
+        weights = numpy.zeros((max_outcome + 1,))
+        
+        zero_index = -self.min_outcome()
+        num_non_negative = len(self.outcomes()) - zero_index
+        weights[:num_non_negative] += self.weights()[zero_index:]
+        weights[1:zero_index+1] += numpy.flip(self.weights()[:zero_index])
+        
+        return Die(weights, 0)
     
-    def relabel(self, relabeling):
-        """
-        relabeling can be one of the following:
-        * An array-like containing relabelings, one for each face in order.
-        * A dict-like mapping old outcomes to new outcomes.
-          Unmapped old outcomes stay the same.
-        * A function mapping old outcomes to new outcomes.
-        """
-        if hasattr(relabeling, 'items'):
-            relabeling = [(relabeling[outcome] if outcome in relabeling else outcome) for outcome in self.outcomes()]
-        elif callable(relabeling):
-            relabeling = [relabeling(outcome) for outcome in self.outcomes()]
-        return Die.mix(*relabeling, mix_weights=self.weights())
-
-    def explode(self, max_explode, *, chance=None, faces=None, outcomes=None):
-        """
-        chance: If supplied, this top fraction of the pmf will explode.
-        faces: If supplied, this many of the top faces will explode.
-        outcomes: If suppled, these outcomes will explode.
-        If neither is supplied, the top single face will explode.
-        """
-        if max_explode < 0:
-            raise ValueError('max_explode cannot be negative.')
-        if max_explode == 0:
-            return self
-        
-        explode_weights = numpy.zeros_like(self.weights())
-        if chance is not None:
-            if chance == 0: return self
-            remaining_explode_weight = chance * self.total_weight()
-            for index, mass in reversed(list(enumerate(self.weights()))):
-                if mass < remaining_explode_weight:
-                    explode_weights[index] = mass
-                    remaining_explode_weight -= mass
-                else:
-                    explode_weights[index] = remaining_explode_weight
-                    break
-        elif faces is not None:
-            if faces == 0: return self
-            explode_weights[-faces:] = self.weights()[-faces:]
-        elif outcomes is not None:
-            explode_faces = numpy.array(outcomes) - self.min_outcome()
-            explode_weights[explode_faces] = self.weights()[explode_faces]
-        else:
-            explode_weights[-1] = self.weights()[-1]
-        
-        non_explode_weights = self.weights() - explode_weights
-        
-        non_explode_die = Die(non_explode_weights, self.min_outcome())._trim()
-        explode_die = Die(explode_weights, self.min_outcome())._trim()
-        explode_die += self.explode(max_explode-1, chance=chance, faces=faces, outcomes=outcomes)
-        
-        mix_weights = [numpy.sum(non_explode_weights), numpy.sum(explode_weights)]
-        
-        return Die.mix(non_explode_die, explode_die, mix_weights=mix_weights)
-    
-    def reroll(self, *, outcomes=None, below=None, above=None, max_reroll=None):
-        """Rerolls the given outcomes."""
-        pmf = numpy.copy(self.pmf())
-        all_outcomes = self.outcomes()
-        reroll_mask = numpy.zeros_like(all_outcomes, dtype=bool)
-        if outcomes is not None: reroll_mask[outcomes - self.min_outcome()] = True
-        if below is not None: reroll_mask[all_outcomes < below] = True
-        if above is not None: reroll_mask[all_outcomes > above] = True
-        
-        reroll_chance_single = numpy.sum(pmf[reroll_mask])
-        stop_chance_single = numpy.sum(pmf[~reroll_mask])
-        
-        if max_rerolls is None:
-            rerollable_factor = 0.0
-            stop_factor = 1.0 / stop_chance_single
-        else:
-            rerollable_factor = numpy.power(reroll_chance_single, max_rerolls)
-            stop_factor = (1.0 - numpy.power(reroll_chance_single, max_rerolls+1)) / stop_chance_single
-        
-        pmf[reroll_mask] *= rerollable_factor
-        pmf[~reroll_mask] *= stop_factor
-        
-        return Die(pmf, self.min_outcome())._trim()
+    abs = __abs__
     
     # Roller wins ties by default. This returns a die that effectively has the given tiebreak mode.
     def tiebreak(self, mode):
@@ -524,7 +467,7 @@ class Die(metaclass=DieType):
             subresults.append(other.repeat_and_sum(die_count))
             die_count_weights.append(die_count_weight)
         
-        subresults = Die._union(*subresults)
+        subresults = Die._align(subresults)
         weights = sum(subresult.weights() * die_count_weight for subresult, die_count_weight in zip(subresults, die_count_weights))
         
         return Die(weights, subresults[0].min_outcome())
@@ -532,21 +475,6 @@ class Die(metaclass=DieType):
     def __rmul__(self, other):
         return Die(other) * self
     
-    def product(self, other):
-        """
-        This multiplies the outcome of the dice. It does not roll additional dice.
-        """
-        other = Die(other)
-        if self.min_outcome < 0 or other.min_outcome < 0:
-            raise NotImplementedError('Multiplication not implemented for non-positive outcomes.')
-        min_outcome = self.min_outcome() * other.min_outcome()
-        max_outcome = self.max_outcome() * other.max_outcome()
-        weights = numpy.zeros((result_max_outcome - resultmin_outcome() + 1))
-        for outcome, mass in zip(other.outcomes(), other.weights()):
-            start_index = outcome * self.min_outcome - resultmin_outcome()
-            weights[start_index:start_index+outcome*len(self.data):outcome] += self.weights() * mass
-        return Die(weights, resultmin_outcome())
-    
     def clip(self, min_outcome=None, max_outcome=None):
         """
         Restricts the outcomes of this die to the range [min_outcome, max_outcome].
@@ -568,26 +496,26 @@ class Die(metaclass=DieType):
     # Repeat, keep, and sum.
     def max(*dice):
         """
-        Returns a Die representing:
         Roll all the dice and take the highest.
+        Dice (or anything castable to a Die) may be provided as a list or as a variable number of arguments.
         """
-        dice = [Die(die) for die in dice]
-        dice_unions = Die._union(*dice)
+        dice = Die._listify_dice(dice)
+        dice_aligned = Die._align(dice)
         cweights = 1.0
-        for die in dice_unions: cweights *= die.cweights()
-        return Die.from_cweights(cweights, dice_unions[0].min_outcome())._trim()
+        for die in dice_aligned: cweights *= die.cweights()
+        return Die.from_cweights(cweights, dice_aligned[0].min_outcome())._trim()
     
     def min(*dice):
         """
-        Returns a Die representing:
         Roll all the dice and take the lowest.
+        Dice (or anything castable to a Die) may be provided as a list or as a variable number of arguments.
         """
-        dice = [Die(die) for die in dice]
+        dice = Die._listify_dice(dice)
         # TODO: use weights
-        dice_unions = Die._union(*dice)
+        dice_aligned = Die._align(dice)
         ccweights = 1.0
-        for die in dice_unions: ccweights *= die.ccweights()
-        return Die.from_ccweights(ccweights, dice_unions[0].min_outcome())._trim()
+        for die in dice_aligned: ccweights *= die.ccweights()
+        return Die.from_ccweights(ccweights, dice_aligned[0].min_outcome())._trim()
     
     def repeat_and_sum(self, num_dice):
         """
@@ -626,51 +554,221 @@ class Die(metaclass=DieType):
         """
         return self._repeater.keep_index(num_dice, index)
 
-    # Comparators. These return scalar floats (which can be cast to Die).
-    # TODO: Return a fraction instead?
+    # Comparators. These return a Die.
     
     def __eq__(self, other):
         """
         Returns the chance this die will roll exactly equal to the other Die.
+        This is in the form of a Die that rolls 1 with that chance, and 0 otherwise.
         """
         other = Die(other)
-        a, b = Die._union(self, other, lcd=False)
-        return numpy.sum(a.pmf() * b.pmf())
+        a, b = Die._align([self, other], lcd=False)
+        n = numpy.dot(a.weights(), b.weights())
+        d = a.total_weight() * b.total_weight()
+        return Die.bernoulli(n, d)
     
     def __ne__(self, other):
         return 1.0 - (self == other)
     
     def __lt__(self, other):
         """
-        Returns the chance this Die will roll < the other Die.        
+        Returns the chance this Die will roll < the other Die.     
+        This is in the form of a Die that rolls 1 with that chance, and 0 otherwise.
         """
         other = Die(other)
         difference = self - other
         if difference.min_outcome() < 0:
-            return numpy.sum(difference.pmf()[:-difference.min_outcome()])
+            n = numpy.sum(difference.weights()[:-difference.min_outcome()])
+            d = difference.total_weight()
+            return Die.bernoulli(n, d)
         else:
-            return 0.0
+            return Die(0)
 
     def __le__(self, other):
         """
-        Returns the chance this Die will roll <= the other Die.        
+        Returns the chance this Die will roll <= the other Die.
+        This is in the form of a Die that rolls 1 with that chance, and 0 otherwise.        
         """
         return self < other + 1
 
     def __gt__(self, other):
         """
-        Returns the chance this Die will roll > the other Die.        
+        Returns the chance this Die will roll > the other Die.
+        This is in the form of a Die that rolls 1 with that chance, and 0 otherwise.
         """
         other = Die(other)
         return other < self
 
     def __ge__(self, other):
         """
-        Returns the chance this Die will roll >= the other Die.        
+        Returns the chance this Die will roll >= the other Die.
+        This is in the form of a Die that rolls 1 with that chance, and 0 otherwise.
         """
         other = Die(other)
         return other <= self
     
+    # Logical operators.
+    # These are only applicable to Bernoulli distributions, i.e. Die that have no outcomes other than 0 and 1.
+    # These return a Die.
+    
+    def __invert__(self):
+        if not self.is_bernoulli():
+            raise ValueError('Logical operators can only be applied to Bernoulli distributions.')
+        return 1 - self
+        
+    def _and(self, other):
+        if not self.is_bernoulli() or not other.bernoulli():
+            raise ValueError('Logical operators can only be applied to Bernoulli distributions.')
+        return (self + other) == 2
+        
+    def __and__(self, other):
+        return self._and(Die(other))
+    
+    def __rand__(self, other):
+        return self._and(Die(other))
+        
+    def _or(self, other):
+        if not self.is_bernoulli() or not other.bernoulli():
+            raise ValueError('Logical operators can only be applied to Bernoulli distributions.')
+        return (self + other) >= 1
+        
+    def __or__(self, other):
+        return self._or(Die(other))
+    
+    def __ror__(self, other):
+        return self._or(Die(other))
+    
+    def _xor(self, other):
+        if not self.is_bernoulli() or not other.bernoulli():
+            raise ValueError('Logical operators can only be applied to Bernoulli distributions.')
+        return (self + other) == 1
+        
+    def __xor__(self, other):
+        return self._xor(Die(other))
+    
+    def __rxor__(self, other):
+        return self._xor(Die(other))
+    
+    # Mixtures.
+    
+    @staticmethod
+    def mix(*dice, mix_weights=None):
+        """
+        Constructs a Die from a mixture of the arguments,
+        equivalent to rolling a die and then choosing one of the arguments
+        based on the resulting face rolled.
+        Dice (or anything castable to a Die) may be provided as a list or as a variable number of arguments.
+        mix_weights: An array of one weight per argument.
+            If not provided, all arguments are mixed uniformly.
+        """
+        dice = Die._listify_dice(dice)
+        dice = Die._align(dice)
+        
+        if mix_weights is None:
+            mix_weights = numpy.ones((len(dice),))
+
+        weights = numpy.zeros_like(dice[0].weights())
+        for die, mix_weight in zip(dice, mix_weights):
+            weights += mix_weight * die.weights()
+        return Die(weights, dice[0].min_outcome())
+    
+    def relabel(self, relabeling):
+        """
+        relabeling can be one of the following:
+        * An array-like containing relabelings, one for each face in order.
+        * A dict-like mapping old outcomes to new outcomes.
+            Unmapped old outcomes stay the same.
+        * A function mapping old outcomes to new outcomes.
+        """
+        if hasattr(relabeling, 'items'):
+            relabeling = [(relabeling[outcome] if outcome in relabeling else outcome) for outcome in self.outcomes()]
+        elif callable(relabeling):
+            relabeling = [relabeling(outcome) for outcome in self.outcomes()]
+
+        return Die.mix(*relabeling, mix_weights=self.weights())
+
+    def explode(self, max_times, outcomes=None):
+        """
+        outcomes: This chooses which outcomes to explode. Options:
+            * A single integer outcome to explode.
+            * A dict whose keys are outcomes and values are chances for that outcome to explode.
+            * An iterable containing outcomes to explode.
+            * If not supplied, the top single outcome will explode.
+        """
+        if max_times < 0:
+            raise ValueError('max_times cannot be negative.')
+        if max_times == 0:
+            return self
+        
+        if outcomes is None:
+            outcomes = self.max_outcome()
+        
+        explode_weights = self._select_weights(outcomes)
+        
+        non_explode_weights = self.weights() - explode_weights
+        
+        non_explode_die = Die(non_explode_weights, self.min_outcome())._trim()
+        explode_die = Die(explode_weights, self.min_outcome())._trim()
+        explode_die += self.explode(max_times-1, outcomes=outcomes)
+        
+        mix_weights = [numpy.sum(non_explode_weights), numpy.sum(explode_weights)]
+        
+        return Die.mix(non_explode_die, explode_die, mix_weights=mix_weights)
+    
+    def reroll(self, *, outcomes=None, max_times=None):
+        """
+        Rerolls the given outcomes.
+        outcomes: Selects which outcomes to reroll. Options:
+            * A single integer outcome to reroll.
+            * A dict whose keys are outcomes and values are chances for that outcome to reroll.
+            * An iterable containing outcomes to reroll.
+        """
+        if outcomes is None:
+            raise TypeError('outcomes to reroll must be provided.')
+        reroll_weights = self._select_weights(outcomes)
+        non_reroll_weights = self.weights() - reroll_weights
+        
+        if max_times is None:
+            if numpy.sum(non_reroll_weights) <= 0.0:
+                raise ZeroDivisionError('This reroll would never terminate.')
+            weights = non_reroll_weights
+        else:
+            total_reroll_weight = numpy.sum(reroll_weights)
+            reroll_chance_single = total_reroll_weight / self.total_weight()
+            rerollable_factor = numpy.power(reroll_chance_single, max_times)
+            stop_factor = (1.0 - numpy.power(reroll_chance_single, max_times+1)) / (1.0 - reroll_chance_single)
+            weights = rerollable_factor * reroll_weights + stop_factor * non_reroll_weights
+        return Die(weights, self.min_outcome())._trim()
+    
+    def combine(*dice, func=None):
+        """
+        Dice (or anything castable to a Die) may be provided as a list or as a variable number of arguments.
+        func should be a function that takes in one outcome for each of the dice and outputs an integer outcome.
+        
+        This method is very flexible but has poor performance since it enumerates all possible joint outcomes.
+        Other methods should be preferred when performance is a concern.
+        """
+        
+        if func is None:
+            raise TypeError('func must be provided')
+        
+        dice = Die._listify_dice(dice)
+        
+        pmf_dict = defaultdict(float)
+        
+        def inner(partial_outcomes, partial_weight, dice):
+            if len(dice) == 0:
+                outcome = func(*partial_outcomes)
+                pmf_dict[outcome] += partial_weight
+            else:
+                for outcome, weight in zip(dice[0].outcomes(), dice[0].weights()):
+                    inner(partial_outcomes + [outcome], partial_weight * weight, dice[1:])
+        
+        inner([], 1.0, dice)
+        
+        return Die(pmf_dict)
+        
+    
     # Random sampling.
     def sample(self, size=None):
         """
@@ -678,21 +776,51 @@ class Die(metaclass=DieType):
         """
         return numpy.random.choice(self.outcomes(), size=size, p=self.pmf())
 
-    # String methods.
+    # Out conversions.
     
     def __str__(self):
         result = ''
-        if self.is_exact():
-            format_string = '%2d, %d\n'
-        else:
-            format_string = '%2d, %f\n'
-        for outcome, weight in zip(self.outcomes(), self.weights()):
-            result += format_string % (outcome, weight)
+        for outcome, weight, p in zip(self.outcomes(), self.weights(), self.pmf()):
+            if self.is_exact():
+                result += '%d, %d/%d, %f\n' % (outcome, weight, self.total_weight(), p)
+            else:
+                result += '%d, %f\n' % (outcome, p)
         return result
+        
+    def __float__(self):
+        if not self.is_bernoulli():
+            raise ValueError('Only Bernoulli distributions may be cast to float.')
+        return float(self.probability(1))
 
     # Helper methods.
     
-    def _union(*dice, lcd=True):
+    @staticmethod
+    def _listify_dice(args):
+        if len(args) == 1 and hasattr(args[0], '__iter__') and not isinstance(args[0], Die):
+            args = args[0]
+        
+        return [Die(arg) for arg in args]
+    
+    def _select_weights(self, outcomes):
+        """
+        Returns an array of weights chosen by the argument.
+        """
+        factors = numpy.zeros_like(self.weights())
+        if numpy.issubdtype(type(outcomes), numpy.integer):
+            factors[outcomes - self._min_outcome] = 1.0 
+        elif hasattr(outcomes, 'items'):
+            # Dict-like.
+            for outcome, factor in outcomes.items():
+                factors[outcome - self._min_outcome] = factor
+        elif hasattr(outcomes, '__iter__'):
+            for outcome in outcomes:
+                factors[outcome - self._min_outcome] = 1.0
+        else:
+            raise TypeError('Invalid type for selecting outcomes.')
+        
+        return factors * self.weights()
+    
+    def _align(dice, lcd=True):
         """ 
         Pads all the dice with zeros so that all have the same min and max outcome.
         If lcd is True, all weights are also multiplied to the least common denominator if all dice are exact.
@@ -700,7 +828,7 @@ class Die(metaclass=DieType):
         """
         min_outcome = min(die.min_outcome() for die in dice)
         max_outcome = max(die.max_outcome() for die in dice)
-        union_len = max_outcome - min_outcome + 1
+        len_align = max_outcome - min_outcome + 1
         
         result_total_weight = 1.0
         if lcd and all(die.is_exact() for die in dice):
@@ -712,7 +840,7 @@ class Die(metaclass=DieType):
         for die in dice:
             weight_factor = result_total_weight / die.total_weight()
             left_dst_index = die.min_outcome() - min_outcome
-            weights = numpy.zeros((union_len,))
+            weights = numpy.zeros((len_align,))
             weights[left_dst_index:left_dst_index + len(die.weights())] = die.weights() * weight_factor
             result.append(Die(weights, min_outcome))
         
